@@ -1,3 +1,5 @@
+# This file is part of cloud-init. See LICENSE file for license information.
+
 from __future__ import print_function
 
 import logging
@@ -223,8 +225,10 @@ class TestKeyValStrings(helpers.TestCase):
 
 class TestGetCmdline(helpers.TestCase):
     def test_cmdline_reads_debug_env(self):
-        os.environ['DEBUG_PROC_CMDLINE'] = 'abcd 123'
-        self.assertEqual(os.environ['DEBUG_PROC_CMDLINE'], util.get_cmdline())
+        with mock.patch.dict("os.environ",
+                             values={'DEBUG_PROC_CMDLINE': 'abcd 123'}):
+            ret = util.get_cmdline()
+        self.assertEqual("abcd 123", ret)
 
 
 class TestLoadYaml(helpers.TestCase):
@@ -384,7 +388,7 @@ class TestReadDMIData(helpers.FilesystemMockingTestCase):
         dmi_name = 'use-dmidecode'
         self._configure_dmidecode_return(dmi_name, dmi_val)
 
-        expected = {'armel': None, 'aarch64': None, 'x86_64': dmi_val}
+        expected = {'armel': None, 'aarch64': dmi_val, 'x86_64': dmi_val}
         found = {}
         # we do not run the 'dmi-decode' binary on some arches
         # verify that anything requested that is not in the sysfs dir
@@ -516,6 +520,7 @@ class TestSubp(helpers.TestCase):
     utf8_invalid = b'ab\xaadef'
     utf8_valid = b'start \xc3\xa9 end'
     utf8_valid_2 = b'd\xc3\xa9j\xc8\xa7'
+    printenv = ['bash', '-c', 'for n in "$@"; do echo "$n=${!n}"; done', '--']
 
     def printf_cmd(self, *args):
         # bash's printf supports \xaa.  So does /usr/bin/printf
@@ -550,7 +555,7 @@ class TestSubp(helpers.TestCase):
     def test_subp_decode_invalid_utf8_replaces(self):
         (out, _err) = util.subp(self.stdin2out, capture=True,
                                 data=self.utf8_invalid)
-        expected = self.utf8_invalid.decode('utf-8', errors='replace')
+        expected = self.utf8_invalid.decode('utf-8', 'replace')
         self.assertEqual(out, expected)
 
     def test_subp_decode_strict_raises(self):
@@ -566,6 +571,29 @@ class TestSubp(helpers.TestCase):
         self.assertEqual(err, data)
         self.assertEqual(out, b'')
 
+    def test_subp_reads_env(self):
+        with mock.patch.dict("os.environ", values={'FOO': 'BAR'}):
+            out, err = util.subp(self.printenv + ['FOO'], capture=True)
+        self.assertEqual('FOO=BAR', out.splitlines()[0])
+
+    def test_subp_env_and_update_env(self):
+        out, err = util.subp(
+            self.printenv + ['FOO', 'HOME', 'K1', 'K2'], capture=True,
+            env={'FOO': 'BAR'},
+            update_env={'HOME': '/myhome', 'K2': 'V2'})
+        self.assertEqual(
+            ['FOO=BAR', 'HOME=/myhome', 'K1=', 'K2=V2'], out.splitlines())
+
+    def test_subp_update_env(self):
+        extra = {'FOO': 'BAR', 'HOME': '/root', 'K1': 'V1'}
+        with mock.patch.dict("os.environ", values=extra):
+            out, err = util.subp(
+                self.printenv + ['FOO', 'HOME', 'K1', 'K2'], capture=True,
+                update_env={'HOME': '/myhome', 'K2': 'V2'})
+
+        self.assertEqual(
+            ['FOO=BAR', 'HOME=/myhome', 'K1=V1', 'K2=V2'], out.splitlines())
+
     def test_returns_none_if_no_capture(self):
         (out, err) = util.subp(self.stdin2out, data=b'', capture=False)
         self.assertEqual(err, None)
@@ -577,4 +605,81 @@ class TestSubp(helpers.TestCase):
         self.assertEqual("/target/my/path/",
                          util.target_path("/target/", "///my/path/"))
 
+
+class TestEncode(helpers.TestCase):
+    """Test the encoding functions"""
+    def test_decode_binary_plain_text_with_hex(self):
+        blob = 'BOOTABLE_FLAG=\x80init=/bin/systemd'
+        text = util.decode_binary(blob)
+        self.assertEqual(text, blob)
+
+
+class TestProcessExecutionError(helpers.TestCase):
+
+    template = ('{description}\n'
+                'Command: {cmd}\n'
+                'Exit code: {exit_code}\n'
+                'Reason: {reason}\n'
+                'Stdout: {stdout}\n'
+                'Stderr: {stderr}')
+    empty_attr = '-'
+    empty_description = 'Unexpected error while running command.'
+
+    def test_pexec_error_indent_text(self):
+        error = util.ProcessExecutionError()
+        msg = 'abc\ndef'
+        formatted = 'abc\n{0}def'.format(' ' * 4)
+        self.assertEqual(error._indent_text(msg, indent_level=4), formatted)
+        self.assertEqual(error._indent_text(msg.encode(), indent_level=4),
+                         formatted.encode())
+        self.assertIsInstance(
+            error._indent_text(msg.encode()), type(msg.encode()))
+
+    def test_pexec_error_type(self):
+        self.assertIsInstance(util.ProcessExecutionError(), IOError)
+
+    def test_pexec_error_empty_msgs(self):
+        error = util.ProcessExecutionError()
+        self.assertTrue(all(attr == self.empty_attr for attr in
+                            (error.stderr, error.stdout, error.reason)))
+        self.assertEqual(error.description, self.empty_description)
+        self.assertEqual(str(error), self.template.format(
+            description=self.empty_description, exit_code=self.empty_attr,
+            reason=self.empty_attr, stdout=self.empty_attr,
+            stderr=self.empty_attr, cmd=self.empty_attr))
+
+    def test_pexec_error_single_line_msgs(self):
+        stdout_msg = 'out out'
+        stderr_msg = 'error error'
+        cmd = 'test command'
+        exit_code = 3
+        error = util.ProcessExecutionError(
+            stdout=stdout_msg, stderr=stderr_msg, exit_code=3, cmd=cmd)
+        self.assertEqual(str(error), self.template.format(
+            description=self.empty_description, stdout=stdout_msg,
+            stderr=stderr_msg, exit_code=str(exit_code),
+            reason=self.empty_attr, cmd=cmd))
+
+    def test_pexec_error_multi_line_msgs(self):
+        # make sure bytes is converted handled properly when formatting
+        stdout_msg = 'multi\nline\noutput message'.encode()
+        stderr_msg = 'multi\nline\nerror message\n\n\n'
+        error = util.ProcessExecutionError(
+            stdout=stdout_msg, stderr=stderr_msg)
+        self.assertEqual(
+            str(error),
+            '\n'.join((
+                '{description}',
+                'Command: {empty_attr}',
+                'Exit code: {empty_attr}',
+                'Reason: {empty_attr}',
+                'Stdout: multi',
+                '        line',
+                '        output message',
+                'Stderr: multi',
+                '        line',
+                '        error message',
+            )).format(description=self.empty_description,
+                      empty_attr=self.empty_attr))
+
 # vi: ts=4 expandtab
